@@ -25,6 +25,10 @@ from tracebloc_package.utils.constants import (
     IMAGE_CLASSIFICATION,
     OBJECT_DETECTION,
     TEXT_CLASSIFICATION,
+    TABULAR_CLASSIFICATION,
+    KEYPOINT_DETECTION,
+    SEMANTIC_SEGMENTATION,
+    YOLO,
 )
 from tracebloc_package.utils.model_upload_utils import task_classes_dict
 
@@ -46,7 +50,7 @@ class User:
         self.__model_upload_params = None
         self.__environment = environment
         self.__url = env_url(self.__environment)
-        if self.__url is None:
+        if self.__url is None:  # pragma: no cover
             text = colored(
                 "\nThe class does not take any arguments. Just run: user = User()",
                 "red",
@@ -61,13 +65,14 @@ class User:
             self.__password = getpass.getpass("Enter your password : ")
         self.__token = self.login()
         self.__ext = ".py"
-        self.__image_size = 224
+        self.__data_shape = 224
         self.__batch_size = 16
         self.__model_path = ""
         self.__framework = TENSORFLOW_FRAMEWORK
         self.__category = IMAGE_CLASSIFICATION
         self.__model_type = ""
         self.__model_id = ""
+        self.__hf_token = ""
         self.__model_name = ""
         self.__model_file_path = ""
         self.__weights_file_path = ""
@@ -77,6 +82,10 @@ class User:
         self.__model_id_llm = None
         self.__loss = None
         self.__message = None
+        self.__utilisation_category = "low"
+        self.__feature_modification = False
+        self.__features = {}
+        self.__table_name = ""
 
     def login(self):
         """Function to get Token for username provided"""
@@ -108,7 +117,7 @@ class User:
                 print("You have been logged out.")
             else:
                 print("Logout Failed. Retry!")
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             print("Logout Failed. Retry!")
 
     def __set_model_val_params(self):
@@ -116,10 +125,36 @@ class User:
         the function set the model validation parameters which will be used for the model file checks to
         validate if model is supported or not
         """
-        self.__image_size = self.__general_checks_obj.image_size
+        # Assign category-specific variables to __data_shape for unified internal usage
+        if self.__general_checks_obj.category in [
+            IMAGE_CLASSIFICATION,
+            OBJECT_DETECTION,
+            SEMANTIC_SEGMENTATION,
+        ]:
+            # CV categories use image_size
+            self.__data_shape = self.__general_checks_obj.image_size
+        elif self.__general_checks_obj.category == KEYPOINT_DETECTION:
+            self.__data_shape = self.__general_checks_obj.image_size
+            self.__features = {
+                "number_of_keypoints": self.__general_checks_obj.num_feature_points
+            }
+        elif self.__general_checks_obj.category == TABULAR_CLASSIFICATION:
+            # Tabular category uses num_feature_points
+            self.__data_shape = self.__general_checks_obj.num_feature_points
+            self.__features = {
+                "number_of_columns": self.__general_checks_obj.num_feature_points
+            }
+        elif self.__general_checks_obj.category == TEXT_CLASSIFICATION:
+            # Text category uses sequence_length
+            self.__data_shape = self.__general_checks_obj.sequence_length
+        else:
+            # Default fallback
+            self.__data_shape = self.__general_checks_obj.image_size
+
         self.__batch_size = self.__general_checks_obj.batch_size
         self.__model_type = self.__general_checks_obj.model_type
         self.__model_id_llm = self.__general_checks_obj.model_id
+        self.__hf_token = self.__general_checks_obj.hf_token
         self.__classes = self.__general_checks_obj.output_classes
         self.__framework = self.__general_checks_obj.framework
         self.__category = self.__general_checks_obj.category
@@ -137,15 +172,10 @@ class User:
             "tmp_model_file_path": self.__general_checks_obj.tmp_file,
             "tmp_dir_path": self.__general_checks_obj.tmp_file_path,
             "model_type": self.__general_checks_obj.model_type,
-            "num_keypoints" : self.__general_checks_obj.num_keypoints,
-            "image_size": self.__image_size,
+            "num_feature_points": self.__general_checks_obj.num_feature_points,
+            "data_shape": self.__data_shape,
             "batch_size": self.__batch_size,
             "weights": self.__weights,
-            "input_shape": (
-                self.__general_checks_obj.input_shape,
-                self.__general_checks_obj.input_shape,
-                3,
-            ),
             "model_path": self.__model_file_path,
             "weights_path": self.__weights_file_path,
             "progress_bar_1": self.__progress_bar,
@@ -181,7 +211,7 @@ class User:
             self.__model = model_checks_generic.model
             self.__general_checks_obj = model_checks_generic
             gc.collect()
-        except:
+        except:  # pragma: no cover
             print_error(
                 text=f"\nUpload failed. There is no model with the name '{self.__model_name}' in your folder '"
                 f"{os.getcwd()}'.",
@@ -218,9 +248,11 @@ class User:
                 self.__model_file_path,
                 self.__weights_file_path,
                 self.__ext,
-            ) = get_paths(path=self.__model_name)
+            ) = get_paths(path=self.__model_name, weights_available=weights)
             # run general model file checks
             self.__run_general_model_checks()
+            if self.__general_checks_obj is None:
+                return None
             self.__set_model_val_params()
 
             # validate and upload model provided
@@ -231,17 +263,27 @@ class User:
             model_check_obj.validate_model_file()
             # get the new model id from server
             self.__model_id = model_check_obj.received_model_name
-            if self.__category == OBJECT_DETECTION:
+            self.__utilisation_category = model_check_obj.utilisation_category
+            if (
+                self.__category == OBJECT_DETECTION
+                and model_check_obj.model_type == YOLO
+            ):
                 self.__loss = model_check_obj.loss
+            else:
+                self.__loss = None
             if self.__model_id == "" or self.__model_id is None:
                 print_error(text=f"'{self.__model_name}' upload Failed.")
                 self.__weights = False
                 return
             else:
                 print_error(text=f"\n'{self.__model_name}' upload successful.\n")
-            gc.collect()
+                remove_tmp_file(
+                    tmp_dir_path=self.__general_checks_obj.tmp_file_path
+                )  # remove file after successful
         except Exception as e:
             raise e
+        finally:
+            gc.collect()
 
     @require_login
     def linkModelDataset(self, datasetId: str):
@@ -263,6 +305,7 @@ class User:
                 return LinkModelDataSet(
                     model_id=self.__model_id_llm,
                     modelId=self.__model_id,
+                    hf_token=self.__hf_token,
                     model=self.__model,
                     modelname=self.__model_name,
                     datasetId=datasetId,
@@ -272,7 +315,7 @@ class User:
                     total_images=self.__total_images,
                     num_classes=self.__num_classes,
                     class_names=self.__class_names,
-                    image_size=self.__image_size,
+                    data_shape=self.__data_shape,
                     batchsize=self.__batch_size,
                     model_path=self.__model_path,
                     url=self.__url,
@@ -281,10 +324,13 @@ class User:
                     model_type=self.__model_type,
                     category=self.__category,
                     loss=self.__loss,
+                    utilisation_category=self.__utilisation_category,
+                    feature_modification=self.__feature_modification,
+                    table_name=self.__table_name,
                 )
             else:
                 return None
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             text = colored("Model Link Failed!", "red")
             print(text, "\n")
 
@@ -302,6 +348,7 @@ class User:
                     "framework": self.__framework,
                     "category": self.__category,
                     "classes": self.__classes,
+                    "match_point": json.dumps(self.__features),
                 },
             )
             if re.status_code == 403 or re.status_code == 400:
@@ -327,24 +374,50 @@ class User:
                     text = colored("Assignment failed!", "red")
                     print(text, "\n")
                     print(f"Dataset '{datasetId}' expected parameters:")
-                    print(
-                        f"classes : {content['datasetClasses']}, shape: {content['datasetShape']}\n"
-                    )
+                    message = f"output_classes : {content['datasetClasses']}, "
+                    if self.__features is not None:
+                        keys_to_match = list(self.__features.keys())
+                    else:
+                        keys_to_match = None
+                    if "matchPoints" in content.keys() and keys_to_match:
+                        for key_to_match in keys_to_match:
+                            expected = content["matchPoints"][key_to_match]["expected"]
+                            if expected and self.__category in [
+                                TABULAR_CLASSIFICATION,
+                                KEYPOINT_DETECTION,
+                            ]:
+                                message += f"num_feature_points : {expected}"
+                            else:
+                                message += f"{key_to_match} : {expected}"
+                    print(f"{message}\n")
                     print(f"'{self.__model_name}' parameters:")
-                    print(
-                        f"classes : {content['outputClass']}, shape: {content['inputShape']}\n"
-                    )
+                    message = f"output_classes : {content['outputClass']}, "
+                    for key_to_match in keys_to_match:
+                        if self.__category in [
+                            TABULAR_CLASSIFICATION,
+                            KEYPOINT_DETECTION,
+                        ]:
+                            message += (
+                                f"num_feature_points : {self.__features[key_to_match]}"
+                            )
+                        else:
+                            message += (
+                                f"{key_to_match} : {self.__features[key_to_match]}"
+                            )
+                    print(f"{message}\n")
                     print(
                         "Please change your model parameters to match the datasets parameters."
                     )
                     return False
                 elif content["status"] == "passed":
-                    text = colored("Assignment successful!", "green")
-                    print(text, "\n")
                     self.__total_images = content["total_images"]
                     self.__num_classes = content["datasetClasses"]
                     self.__class_names = content["class_names"]
                     self.__totalDatasetSize = getImagesCount(self.__class_names)
+                    self.__feature_modification = content["allow_feature_modification"]
+                    self.__table_name = content["table_name"]
+                    text = colored("Assignment successful!", "green")
+                    print(text, "\n")
                     print(
                         f"\n \033[1mDataset Parameters\033[0m\n\n",
                         f"datasetId: {datasetId}\n",
@@ -357,7 +430,7 @@ class User:
                 text = colored(f"Error Occurred. Linking Failed!", "red")
                 print(text)
                 return False
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             if self.__environment != "" or self.__environment != "production":
                 print(f"Error occurred while setting variables as {e}")
             text = colored(f"Communication Fail Error!", "red")
@@ -385,5 +458,5 @@ class User:
         print("Link uploaded model with a dataset.\n")
 
         rich.print(
-            "For more information check the [link=https://docs.tracebloc.io/user-uploadModel]docs[/link]"
+            "For more information check the [link=https://docs.tracebloc.io/join-use-case/start-training]docs[/link]"
         )
